==> wmconfig-1.2.4.tar.bz2: wmconfig-1.2.4/kde2.c <==
/*
 * Copyright (C) 2002
 *	Tommy Scheunemann <net@arrishq.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>
#include <limits.h>

#include "wmconfig.h"
#include "prototypes.h"
#include "package.h"

/* define default output dir
 if the directory exists and there are
 contents, they will be replaced with
 Wmconfig entries
*/

#define KDE_DIR ".kde/share/applnk/wmconfig"

/* define default icon - used for menu descriptions */

#define KDE_MENU_ICON "package"
extern const char *root_menu_name;

/* KDE2 and KDE3 output function */

/* Output application launchers */

static void create_kdelnk(const char *file, struct package *app)
{
 FILE *f;
 if (app == (struct package *)NULL) {
  return;
 }
 if (app->exec != NULL) {
  char *p;
  p = strrchr(app->exec, '&');
  if (p != NULL)
   *p = '\0';
 }
 if (file == NULL || app->exec == NULL || app->restart)
  return;
 f = fopen(file, "w");
 if (f == (FILE *) NULL) {
  fprintf(stderr,"Could not create file %s\n", file);
  perror("Error code");
  return;
 }
 fprintf(f, "[Desktop Entry]\n");
 if (app->name != NULL)
  fprintf(f, "Name=%s\n", app->name);
 if (app->icon != NULL)
  fprintf(f, "Icon=%s\n", app->icon);
 if (app->mini_icon != NULL)
  fprintf(f, "MiniIcon=%s\n", app->mini_icon);
 if (app->description != NULL)
  fprintf(f, "Comment=%s\n", app->description);
 fprintf(f, "Exec=%s\n", app->exec);
 fprintf(f, "Type=Application\n");
 if (app->mimetype != NULL)
  fprintf(f, "MimeType=%s;\n", app->mimetype);
 if (app->terminal != NULL)
  fprintf(f, "Terminal=true\n");
 fclose(f);
}

/* Create the directories required for the submenus */

static void make_directory_path(const char *path)
{
 char buf[PATH_MAX];
 int pi=0;
 int bi=0;
 while( path[pi] != '\0' ) {
  buf[bi] = path[pi];
  bi++ ; pi++ ;
  buf[bi] = '\0';
  if ( path[pi] == '\0' || path[pi] == '/' ) {
   mkdir(buf, 0700);
  }
 }
}

/* Create directory descriptions */

static void create_kde2_menu_directory(const char *directory, const char *name)
{
 char dot_directory[PATH_MAX+1];
 FILE *f;
 make_directory_path(directory);
 snprintf(dot_directory, PATH_MAX, "%s/.directory", directory);
 f = fopen(dot_directory, "w");
 if (f == (FILE *) NULL) {
  fprintf(stderr,"Could not create file %s\n", dot_directory);
  perror("Error code");
  return;
 }
 fprintf(f, "[Desktop Entry]\n");
 fprintf(f, "Icon=%s\n", KDE_MENU_ICON);
 fprintf(f, "Name=%s\n", name);
 fclose(f);
}

/* Create the directory and fill it */

static void make_dir(struct group *root, int level, const char *dir_name)
{
 struct item *item;
 char c_tmp[PATH_MAX];
 if (root == (struct group *)NULL)
  return;
 item = root->items;
 while (item->type != 0) {
  if (item->type == ITEM_MENU) {
   struct group *tmp;
   tmp = (struct group *)item->data;
   snprintf(c_tmp, PATH_MAX, "%s/%s", single_string(dir_name), single_string(tmp->name) );
   create_kde2_menu_directory(c_tmp, tmp->name);
  } else if (item->type == ITEM_APP) {
   struct package *app;
   app = (struct package *)item->data;
   if (app->name && app->exec) {
    snprintf(c_tmp, PATH_MAX, "%s/%s.desktop", single_string(dir_name), single_string(app->name) );
    create_kdelnk(c_tmp, app);
   }
  }
  item++;
 }
 item = root->items;
 while (item->type != 0) {
  if (item->type == ITEM_MENU) {
   struct group *tmp;
   tmp = (struct group *)item->data;
   snprintf(c_tmp, PATH_MAX, "%s/%s", dir_name, tmp->name);
   make_dir(item->data, level+1, c_tmp);
  }
  item++;
 }
}
static void clean_dir(const char *dir_name)
{
 struct dirent **namelist;
 int nr_files;
 int i;
 if (dir_name == NULL)
  return;
 nr_files = scandir(dir_name, &namelist, NULL, alphasort);
 for (i = 0; i < nr_files; i++) {
  if ( (strcmp(namelist[i]->d_name, ".") != 0) &&
  (strcmp(namelist[i]->d_name, "..") != 0) ) {
   struct stat st;
   char tmp[PATH_MAX];
   int retval;
   snprintf(tmp, PATH_MAX, "%s/%s", dir_name, namelist[i]->d_name);
   if (stat(tmp, &st) != 0) {
    fprintf(stderr, "Could not stat %s\n", tmp);
    return;
   }
   if (S_ISDIR(st.st_mode)) {
    clean_dir(tmp);
    retval = rmdir(tmp);
   } else {
    retval = unlink(tmp);
   }
   if (retval != 0) {
    perror("Could not remove");
    return;
   }
  }
  free(namelist[i]);
 }
 if (nr_files && namelist)
  free(namelist);	    
}
void output_kde2(struct group *root)
{
 struct stat st;
 char root_dir[PATH_MAX];
 struct passwd *pw;
 setpwent();
 pw = getpwuid(getuid());
 if (pw == (struct passwd *)NULL) {
  fprintf(stderr, "Could not find out who you are (getpwnam failed)!\n");
  return;
 }
 endpwent();
 snprintf(root_dir, PATH_MAX, "%s/%s", pw->pw_dir, KDE_DIR);
 if (stat(root_dir, &st) == 0) {
  if (!S_ISDIR(st.st_mode)) {
   fprintf(stderr, "Error: %s exists but it is not a directory\n",
   root_dir);
   return;
  }
  clean_dir(root_dir);
  rmdir(root_dir);
 }
 create_kde2_menu_directory(root_dir, DEFAULT_ROOT_NAME);
 make_dir(root, 0, root_dir);
}

==> wmconfig-1.2.5.tar.bz2: wmconfig-1.2.5/kde2.c <==
/*
 * Copyright (C) 2002
 *	Tommy Scheunemann <net@arrishq.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>
#include <limits.h>

#include "wmconfig.h"
#include "prototypes.h"
#include "package.h"

/* define default output dir
 if the directory exists and there are
 contents, they will be replaced with
 Wmconfig entries
*/

/* Default Output Dir */
#define KDE_DIR		".kde/share/applnk/wmconfig"

/* Icon for the menus */
#define KDE_MENU_ICON	"package.xpm"

extern const char *root_menu_name;
extern const char *output_directory;
extern unsigned int flags;

/* KDE2 and KDE3 output function */

/* Output application launchers */

static void create_kdelnk(const char *file, struct package *app)
{
 FILE *f;
 if (app == (struct package *)NULL) {
  return;
 }
 if (app->exec != NULL) {
  char *p;
  p = strrchr(app->exec, '&');
  if (p != NULL)
   *p = '\0';
 }
 if (file == NULL || app->exec == NULL || app->restart)
  return;
 f = fopen(file, "w");
 if (f == (FILE *) NULL) {
  fprintf(stderr,"Could not create file %s\n", file);
  perror("Error code");
  return;
 }
 fprintf(f, "[Desktop Entry]\n");
 if (app->name != NULL)
  fprintf(f, "Name=%s\n", app->name);
 if (app->icon != NULL)
  fprintf(f, "Icon=%s\n", app->icon);
 if (app->mini_icon != NULL)
  fprintf(f, "MiniIcon=%s\n", app->mini_icon);
 if (app->description != NULL)
  fprintf(f, "Comment=%s\n", app->description);
 fprintf(f, "Exec=%s\n", app->exec);
 fprintf(f, "Type=Application\n");
 if (app->mimetype != NULL)
  fprintf(f, "MimeType=%s;\n", app->mimetype);
 if (app->terminal != NULL)
  fprintf(f, "Terminal=true\n");
 fclose(f);
}

/* Create the directories required for the submenus */

static void make_directory_path(const char *path)
{
 char buf[PATH_MAX];
 int pi=0;
 int bi=0;
 while( path[pi] != '\0' ) {
  buf[bi] = path[pi];
  bi++ ; pi++ ;
  buf[bi] = '\0';
  if ( path[pi] == '\0' || path[pi] == '/' ) {
   mkdir(buf, 0700);
  }
 }
}

/* Create directory descriptions */

static void create_kde2_menu_directory(const char *directory, const char *name)
{
 char dot_directory[PATH_MAX+1];
 FILE *f;
 make_directory_path(directory);
 snprintf(dot_directory, PATH_MAX, "%s/.directory", directory);
 f = fopen(dot_directory, "w");
 if (f == (FILE *) NULL) {
  fprintf(stderr,"Could not create file %s\n", dot_directory);
  perror("Error code");
  return;
 }
 fprintf(f, "[Desktop Entry]\n");
 fprintf(f, "Icon=%s\n", KDE_MENU_ICON);
 fprintf(f, "Name=%s\n", name);
 fclose(f);
}

/* Create the directory and fill it */

static void make_dir(struct group *root, int level, const char *dir_name)
{
 struct item *item;
 char c_tmp[PATH_MAX];
 if (root == (struct group *)NULL)
  return;
 item = root->items;
 while (item->type != 0) {
  if (item->type == ITEM_MENU) {
   struct group *tmp;
   tmp = (struct group *)item->data;
   snprintf(c_tmp, PATH_MAX, "%s/%s", single_string(dir_name), single_string(tmp->name) );
   create_kde2_menu_directory(c_tmp, tmp->name);
  } else if (item->type == ITEM_APP) {
   struct package *app;
   app = (struct package *)item->data;
   if (app->name && app->exec) {
    snprintf(c_tmp, PATH_MAX, "%s/%s.desktop", single_string(dir_name), single_string(app->name) );
    create_kdelnk(c_tmp, app);
   }
  }
  item++;
 }
 item = root->items;
 while (item->type != 0) {
  if (item->type == ITEM_MENU) {
   struct group *tmp;
   tmp = (struct group *)item->data;
   snprintf(c_tmp, PATH_MAX, "%s/%s", dir_name, tmp->name);
   make_dir(item->data, level+1, c_tmp);
  }
  item++;
 }
}
static void clean_dir(const char *dir_name)
{
 struct dirent **namelist;
 int nr_files;
 int i;
 if (dir_name == NULL)
  return;
 nr_files = scandir(dir_name, &namelist, NULL, alphasort);
 for (i = 0; i < nr_files; i++) {
  if ( (strcmp(namelist[i]->d_name, ".") != 0) &&
  (strcmp(namelist[i]->d_name, "..") != 0) ) {
   struct stat st;
   char tmp[PATH_MAX];
   int retval;
   snprintf(tmp, PATH_MAX, "%s/%s", dir_name, namelist[i]->d_name);
   if (stat(tmp, &st) != 0) {
    fprintf(stderr, "Could not stat %s\n", tmp);
    return;
   }
   if (S_ISDIR(st.st_mode)) {
    clean_dir(tmp);
    retval = rmdir(tmp);
   } else {
    retval = unlink(tmp);
   }
   if (retval != 0) {
    perror("Could not remove");
    return;
   }
  }
  free(namelist[i]);
 }
 if (nr_files && namelist)
  free(namelist);	    
}
void output_kde2(struct group *root)
{
 struct stat st;
 char root_dir[PATH_MAX];
 struct passwd *pw;
 setpwent();
 pw = getpwuid(getuid());
 if (pw == (struct passwd *)NULL) {
  fprintf(stderr, "Could not find out who you are (getpwnam failed)!\n");
  return;
 }
 endpwent();

 if (output_directory == NULL) {
  snprintf(root_dir, PATH_MAX, "%s/%s", pw->pw_dir, KDE_DIR);
 } else {
  snprintf(root_dir, PATH_MAX, "%s", output_directory);
 }

 if (stat(root_dir, &st) == 0) {
  if (!S_ISDIR(st.st_mode)) {
   fprintf(stderr, "Error: %s exists but it is not a directory\n",
   root_dir);
   return;
  }
  clean_dir(root_dir);
  rmdir(root_dir);
 }
 create_kde2_menu_directory(root_dir, DEFAULT_ROOT_NAME);
 make_dir(root, 0, root_dir);
}
